@@ -90,7 +90,7 @@ class BasicLinkerIsCompilerMixin:
         raise mesonlib.EnvironmentException(
             'Linker {} does not support link_whole'.format(self.id))
 
-    def get_allow_undefined_args(self) -> typing.List[str]:
+    def get_allow_undefined_link_args(self) -> typing.List[str]:
         raise mesonlib.EnvironmentException(
             'Linker {} does not support allow undefined'.format(self.id))
 
@@ -121,3 +121,12 @@ class BasicLinkerIsCompilerMixin:
                          rpath_paths: str, build_rpath: str,
                          install_rpath: str) -> typing.List[str]:
         return []
+
+    def get_linker_debug_crt_args(self) -> typing.List[str]:
+        return []
+
+    def get_asneeded_args(self) -> typing.List[str]:
+        return []
+
+    def get_buildtype_linker_args(self, buildtype: str) -> typing.List[str]:
+        return []
