@@ -13,11 +13,14 @@
 # limitations under the License.
 
 import subprocess, os.path
+import typing
 
 from ..mesonlib import EnvironmentException, MachineChoice, Popen_safe
-
 from .compilers import Compiler, rust_buildtype_args, clike_debug_args
 
+if typing.TYPE_CHECKING:
+    from ..environment import Environment  # noqa: F401
+
 rust_optimization_args = {'0': [],
                           'g': ['-C', 'opt-level=0'],
                           '1': ['-C', 'opt-level=1'],
@@ -27,9 +30,9 @@ rust_optimization_args = {'0': [],
                           }
 
 class RustCompiler(Compiler):
-    def __init__(self, exelist, version, for_machine: MachineChoice, is_cross, exe_wrapper=None):
+    def __init__(self, exelist, version, for_machine: MachineChoice, is_cross, exe_wrapper=None, **kwargs):
         self.language = 'rust'
-        super().__init__(exelist, version, for_machine)
+        super().__init__(exelist, version, for_machine, **kwargs)
         self.exe_wrapper = exe_wrapper
         self.id = 'rustc'
         self.is_cross = is_cross
@@ -77,9 +80,6 @@ class RustCompiler(Compiler):
     def get_buildtype_args(self, buildtype):
         return rust_buildtype_args[buildtype]
 
-    def build_rpath_args(self, build_dir, from_dir, rpath_paths, build_rpath, install_rpath):
-        return self.build_unix_rpath_args(build_dir, from_dir, rpath_paths, build_rpath, install_rpath)
-
     def get_sysroot(self):
         cmd = self.exelist + ['--print', 'sysroot']
         p, stdo, stde = Popen_safe(cmd)
@@ -102,8 +102,5 @@ class RustCompiler(Compiler):
 
         return parameter_list
 
-    def get_buildtype_linker_args(self, build_type):
-        return []
-
     def get_std_exe_link_args(self):
         return []
