@@ -14,6 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import typing
 import itertools
 import os
 import subprocess
@@ -108,20 +109,13 @@ def setup_commands(optbackend):
     compile_commands, clean_commands, test_commands, install_commands, \
         uninstall_commands = get_backend_commands(backend, do_debug)
 
-def get_relative_files_list_from_dir(fromdir):
-    paths = []
-    for (root, _, files) in os.walk(fromdir):
-        reldir = os.path.relpath(root, start=fromdir)
-        for f in files:
-            path = os.path.join(reldir, f).replace('\\', '/')
-            if path.startswith('./'):
-                path = path[2:]
-            paths.append(path)
-    return paths
+def get_relative_files_list_from_dir(fromdir: Path) -> typing.List[Path]:
+    return [file.relative_to(fromdir) for file in fromdir.rglob('*') if file.is_file()]
 
-def platform_fix_name(fname, compiler, env):
+def platform_fix_name(fname: str, compiler, env) -> str:
     # canonicalize compiler
-    if compiler in {'clang-cl', 'intel-cl'}:
+    if (compiler in {'clang-cl', 'intel-cl'} or
+       (env.machines.host.is_windows() and compiler == 'pgi')):
         canonical_compiler = 'msvc'
     else:
         canonical_compiler = compiler
@@ -199,35 +193,36 @@ def platform_fix_name(fname, compiler, env):
 
     return fname
 
-def validate_install(srcdir, installdir, compiler, env):
+def validate_install(srcdir: str, installdir: Path, compiler, env) -> str:
     # List of installed files
-    info_file = os.path.join(srcdir, 'installed_files.txt')
+    info_file = Path(srcdir) / 'installed_files.txt'
+    installdir = Path(installdir)
     # If this exists, the test does not install any other files
-    noinst_file = 'usr/no-installed-files'
-    expected = {}
+    noinst_file = Path('usr/no-installed-files')
+    expected = {}  # type: typing.Dict[Path, bool]
     ret_msg = ''
     # Generate list of expected files
-    if os.path.exists(os.path.join(installdir, noinst_file)):
+    if (installdir / noinst_file).is_file():
         expected[noinst_file] = False
-    elif os.path.exists(info_file):
-        with open(info_file) as f:
+    elif info_file.is_file():
+        with info_file.open() as f:
             for line in f:
                 line = platform_fix_name(line.strip(), compiler, env)
                 if line:
-                    expected[line] = False
+                    expected[Path(line)] = False
     # Check if expected files were found
     for fname in expected:
-        file_path = os.path.join(installdir, fname)
-        if os.path.exists(file_path) or os.path.islink(file_path):
+        file_path = installdir / fname
+        if file_path.is_file() or file_path.is_symlink():
             expected[fname] = True
     for (fname, found) in expected.items():
         if not found:
-            ret_msg += 'Expected file {0} missing.\n'.format(fname)
+            ret_msg += 'Expected file {} missing.\n'.format(fname)
     # Check if there are any unexpected files
     found = get_relative_files_list_from_dir(installdir)
     for fname in found:
         if fname not in expected:
-            ret_msg += 'Extra file {0} found.\n'.format(fname)
+            ret_msg += 'Extra file {} found.\n'.format(fname)
     if ret_msg != '':
         ret_msg += '\nInstall dir contents:\n'
         for i in found:
@@ -340,7 +335,7 @@ def pass_libdir_to_test(dirname):
         return False
     if '38 libdir must be inside prefix' in dirname:
         return False
-    if '196 install_mode' in dirname:
+    if '195 install_mode' in dirname:
         return False
     return True
 
@@ -444,12 +439,12 @@ def _run_test(testdir, test_build_dir, install_dir, extra_args, compiler, backen
     return TestResult(validate_install(testdir, install_dir, compiler, builddata.environment),
                       BuildStep.validate, stdo, stde, mesonlog, gen_time, build_time, test_time)
 
-def gather_tests(testdir: Path):
-    tests = [t.name for t in testdir.glob('*')]
-    tests = [t for t in tests if not t.startswith('.')] # Filter non-tests files (dot files, etc)
-    testlist = [(int(t.split()[0]), t) for t in tests]
-    testlist.sort()
-    tests = [testdir / t[1] for t in testlist]
+def gather_tests(testdir: Path) -> typing.List[Path]:
+    test_names = [t.name for t in testdir.glob('*') if t.is_dir()]
+    test_names = [t for t in test_names if not t.startswith('.')] # Filter non-tests files (dot files, etc)
+    test_nums = [(int(t.split()[0]), t) for t in test_names]
+    test_nums.sort()
+    tests = [testdir / t[1] for t in test_nums]
     return tests
 
 def have_d_compiler():
@@ -528,6 +523,10 @@ def skippable(suite, test):
     if test.endswith('14 fortran links c'):
         return True
 
+    # Blocks are not supported on all compilers
+    if test.endswith('29 blocks'):
+        return True
+
     # No frameworks test should be skipped on linux CI, as we expect all
     # prerequisites to be installed
     if mesonlib.is_linux():
@@ -597,7 +596,22 @@ def should_skip_rust() -> bool:
             return True
     return False
 
-def detect_tests_to_run():
+def detect_tests_to_run(only: typing.List[str]) -> typing.List[typing.Tuple[str, typing.List[Path], bool]]:
+    """
+    Parameters
+    ----------
+    only: list of str, optional
+        specify names of tests to run
+
+    Returns
+    -------
+    gathered_tests: list of tuple of str, list of pathlib.Path, bool
+        tests to run
+    """
+
+    skip_fortran = not(shutil.which('gfortran') or shutil.which('flang') or
+                       shutil.which('pgfortran') or shutil.which('ifort'))
+
     # Name, subdirectory, skip condition.
     all_tests = [
         ('cmake', 'cmake', not shutil.which('cmake') or (os.environ.get('compiler') == 'msvc2015' and under_ci)),
@@ -619,7 +633,7 @@ def detect_tests_to_run():
         ('d', 'd', backend is not Backend.ninja or not have_d_compiler()),
         ('objective c', 'objc', backend not in (Backend.ninja, Backend.xcode) or not have_objc_compiler()),
         ('objective c++', 'objcpp', backend not in (Backend.ninja, Backend.xcode) or not have_objcpp_compiler()),
-        ('fortran', 'fortran', backend is not Backend.ninja or not shutil.which('gfortran')),
+        ('fortran', 'fortran', skip_fortran or backend != Backend.ninja),
         ('swift', 'swift', backend not in (Backend.ninja, Backend.xcode) or not shutil.which('swiftc')),
         ('cuda', 'cuda', backend not in (Backend.ninja, Backend.xcode) or not shutil.which('nvcc')),
         ('python3', 'python3', backend is not Backend.ninja),
@@ -627,18 +641,24 @@ def detect_tests_to_run():
         ('fpga', 'fpga', shutil.which('yosys') is None),
         ('frameworks', 'frameworks', False),
         ('nasm', 'nasm', False),
+        ('wasm', 'wasm', shutil.which('emcc') is None or backend is not Backend.ninja),
     ]
+
+    if only:
+        names = [t[0] for t in all_tests]
+        ind = [names.index(o) for o in only]
+        all_tests = [all_tests[i] for i in ind]
     gathered_tests = [(name, gather_tests(Path('test cases', subdir)), skip) for name, subdir, skip in all_tests]
     return gathered_tests
 
-def run_tests(all_tests, log_name_base, failfast, extra_args):
+def run_tests(all_tests, log_name_base, failfast: bool, extra_args):
     global logfile
     txtname = log_name_base + '.txt'
     with open(txtname, 'w', encoding='utf-8', errors='ignore') as lf:
         logfile = lf
         return _run_tests(all_tests, log_name_base, failfast, extra_args)
 
-def _run_tests(all_tests, log_name_base, failfast, extra_args):
+def _run_tests(all_tests, log_name_base, failfast: bool, extra_args):
     global stop, executor, futures, system_compiler
     xmlname = log_name_base + '.xml'
     junit_root = ET.Element('testsuites')
@@ -744,8 +764,8 @@ def _run_tests(all_tests, log_name_base, failfast, extra_args):
                 stdeel = ET.SubElement(current_test, 'system-err')
                 stdeel.text = result.stde
 
-        if failfast and failing_tests > 0:
-            break
+            if failfast and failing_tests > 0:
+                break
 
     print("\nTotal configuration time: %.2fs" % conf_time)
     print("Total build time: %.2fs" % build_time)
@@ -753,19 +773,14 @@ def _run_tests(all_tests, log_name_base, failfast, extra_args):
     ET.ElementTree(element=junit_root).write(xmlname, xml_declaration=True, encoding='UTF-8')
     return passing_tests, failing_tests, skipped_tests
 
-def check_file(fname):
-    linenum = 1
-    with open(fname, 'rb') as f:
-        lines = f.readlines()
+def check_file(file: Path):
+    lines = file.read_bytes().split(b'\n')
     tabdetector = re.compile(br' *\t')
-    for line in lines:
+    for i, line in enumerate(lines):
         if re.match(tabdetector, line):
-            print("File %s contains a tab indent on line %d. Only spaces are permitted." % (fname, linenum))
-            sys.exit(1)
-        if b'\r' in line:
-            print("File %s contains DOS line ending on line %d. Only unix-style line endings are permitted." % (fname, linenum))
-            sys.exit(1)
-        linenum += 1
+            raise SystemExit("File {} contains a tab indent on line {:d}. Only spaces are permitted.".format(file, i + 1))
+        if line.endswith(b'\r'):
+            raise SystemExit("File {} contains DOS line ending on line {:d}. Only unix-style line endings are permitted.".format(file, i + 1))
 
 def check_format():
     check_suffixes = {'.c',
@@ -787,20 +802,21 @@ def check_format():
                       '.build',
                       '.md',
                       }
-    for (root, _, files) in os.walk('.'):
+    for (root, _, filenames) in os.walk('.'):
         if '.dub' in root: # external deps are here
             continue
         if '.pytest_cache' in root:
             continue
         if 'meson-logs' in root or 'meson-private' in root:
             continue
-        for fname in files:
-            if os.path.splitext(fname)[1].lower() in check_suffixes:
-                bn = os.path.basename(fname)
-                if bn == 'sitemap.txt' or bn == 'meson-test-run.txt':
+        if '.eggs' in root or '_cache' in root:  # e.g. .mypy_cache
+            continue
+        for fname in filenames:
+            file = Path(fname)
+            if file.suffix.lower() in check_suffixes:
+                if file.name in ('sitemap.txt', 'meson-test-run.txt'):
                     continue
-                fullname = os.path.join(root, fname)
-                check_file(fullname)
+                check_file(root / file)
 
 def check_meson_commands_work():
     global backend, compile_commands, test_commands, install_commands
@@ -859,6 +875,9 @@ if __name__ == '__main__':
                         choices=backendlist)
     parser.add_argument('--failfast', action='store_true',
                         help='Stop running if test case fails')
+    parser.add_argument('--no-unittests', action='store_true',
+                        help='Not used, only here to simplify run_tests.py')
+    parser.add_argument('--only', help='name of test(s) to run', nargs='+')
     options = parser.parse_args()
     setup_commands(options.backend)
 
@@ -869,7 +888,7 @@ if __name__ == '__main__':
     check_format()
     check_meson_commands_work()
     try:
-        all_tests = detect_tests_to_run()
+        all_tests = detect_tests_to_run(options.only)
         (passing_tests, failing_tests, skipped_tests) = run_tests(all_tests, 'meson-test-run', options.failfast, options.extra_args)
     except StopException:
         pass
@@ -884,9 +903,9 @@ if __name__ == '__main__':
             except UnicodeError:
                 print(l.encode('ascii', errors='replace').decode(), '\n')
     for name, dirs, _ in all_tests:
-        dirs = (x.name for x in dirs)
-        for k, g in itertools.groupby(dirs, key=lambda x: x.split()[0]):
+        dir_names = (x.name for x in dirs)
+        for k, g in itertools.groupby(dir_names, key=lambda x: x.split()[0]):
             tests = list(g)
             if len(tests) != 1:
                 print('WARNING: The %s suite contains duplicate "%s" tests: "%s"' % (name, k, '", "'.join(tests)))
-    sys.exit(failing_tests)
+    raise SystemExit(failing_tests)
